############################################
# Regression Script for simdata

import os, time

os.system('rm -rf 30dor.image')

startTime = time.time()
startProc = time.clock()

print '-- Starting simdata test'
project="ghii2"

# Clear out results from previous runs.
os.system('rm -rf '+project+'*')

import datetime
datestring = datetime.datetime.isoformat(datetime.datetime.today())
outfile    = project + '.' + datestring + '.log'
logfile    = open(outfile, 'w')

print 'Writing regression output to ' + outfile + " - in case of failure that is the file we will want to see\n"
print >>logfile, "CASA simdata regression test:\n"

# configs are in the repository
l=locals() 
if not l.has_key("repodir"): 
    repodir=os.getenv("CASAPATH").split(' ')[0]

print "CASA version "+casa['build']['version']+" v"+casa['build']['number']+" running on"
print >>logfile, "CASA version "+casa['build']['version']+" v"+casa['build']['number']+" running on"
uname=os.popen("uname -a").read()
print "["+uname+"]\n"
print >>logfile,"["+uname+"]\n\n"


datadir=repodir+"/data/regression/simdata/"
# local operation
datadir="./"
cfgdir=repodir+"/data/alma/simmos/"
if not os.path.exists(datadir+"30dor.old.fits"):
    print "You must have the file 30dor_new.fits"
else:
    importfits(fitsimage=datadir+"30dor.old.fits",imagename="30dor.image")
        
    default("simdata")
    project="ghii2"
    
    cl.done()
    cl.addcomponent(dir="J2000 05h18m48.586s -68d42m00.05s",flux=0.5,freq="650GHz")
    cl.rename(project+".cl")
    cl.done()
    
    modifymodel=True # but only brightness
    skymodel="30dor.image"
    inbright=".1"
    incell=""
    incenter="652GHz"   
    inwidth="4GHz" 
    indirection=""
    
    setpointings=True
    integration="10s"
    totaltime="3600s"
    direction="J2000 05h18m48.0s -68d41m59s"
    mapsize="10arcsec"  # input is 16.3, relmargin=0.5
    pointingspacing="4.5arcsec"
    
    predict=True
    complist=project+".cl"
    refdate="2012/06/21/03:25:00"
    antennalist=cfgdir+"alma.out05.cfg"
    
    thermalnoise="tsys-atm"
    
    image=True
    vis="$project.noisy.ms"
    cell="0.05arcsec"
    niter=15000
    threshold="10mJy"
    weighting="briggs"
    imsize=[400,400]
    mask=[129,168,255,286]
    
    analyze=True
    overwrite=True
    verbose=False
    
    if not l.has_key('interactive'): interactive=False
    if interactive:
        graphics="both"
    else:
        graphics="file"
    
    go()
    
    endTime = time.time()
    endProc = time.clock()
    
    # Regression

    project = project+'/'+project
    
    test_name = """simdata observation of 30 Doradus"""
    
    ia.open(project + '.image')
    hii_stats=ia.statistics(verbose=False,list=False)
    ia.close()
        
    ia.open(project + '.diff')
    hiidiff_stats=ia.statistics(verbose=False,list=False)
    ia.close()
    # 3.1 # 3.2
    refstats = { 'sum': 790, #756., #830
                 'max': 0.68, #0.635, # 0.70
                 'min': -0.072, #0.0673 # .076
                 'rms': 0.063, #0.0579, # .0664
                 'sigma': 0.062 } #0.057 # .065
    
    diffstats = {'sum': 3200, #3117., # 3235
                 'max': 0.27, #0.392, #0.253
#                 'min': -0.01, #-0.01, 
                 'rms': 0.05, #0.052, # .053
                 'sigma': 0.027 } #0.029 #.028
    
    
    reftol   = {'sum':  0.1,
                'max':  0.15,
                'min':  0.15,
                'rms':  0.15,
                'sigma': 0.15}
    
    
    loghdr = "********** Regression ************"
    
    print >> logfile, loghdr
    
    # more info
    ms.open(project+".ms")
    print >> logfile, "Noiseless MS, amp stats:"
    print >> logfile, ms.statistics('DATA','amp')
    print >> logfile, "Noiseless MS, phase stats:"
    print >> logfile, ms.statistics('DATA','phase')
    ms.close()
    
    print hii_stats
    regstate = True
    rskes = refstats.keys()
    rskes.sort()
    for ke in rskes:
        adiff=abs(hii_stats[ke][0] - refstats[ke])/abs(refstats[ke])
        if adiff < reftol[ke]:
            print >> logfile, "* Passed %-5s image test, got % -11.5g expected % -11.5g." % (ke, hii_stats[ke][0], refstats[ke])
        else:
            print >> logfile, "* FAILED %-5s image test, got % -11.5g instead of % -11.5g." % (ke, hii_stats[ke][0], refstats[ke])
            regstate = False
    
    rskes = diffstats.keys()
    rskes.sort()
    for ke in rskes:
        adiff=abs(hiidiff_stats[ke][0] - diffstats[ke])/abs(diffstats[ke])
        if adiff < reftol[ke]:
            print >> logfile, "* Passed %-5s  diff test, got % -11.5g expected % -11.5g." % (ke, hiidiff_stats[ke][0], diffstats[ke])
        else:
            print >> logfile, "* FAILED %-5s  diff test, got % -11.5g instead of % -11.5g." % (ke, hiidiff_stats[ke][0], diffstats[ke])
            regstate = False
            
    
    print >> logfile,'---'
    if regstate:
        print >> logfile, 'Passed',
        print '\nPassed simulation test.  You may delete '+project+'* now.',
    else:
        print >> logfile, 'FAILED',
        print '\nFAILED - please look at '+outfile+" for details"
    print >> logfile, 'regression test for simdata of 30 Doradus.'
    print >>logfile,'---'
        
    print >>logfile,''
    print >>logfile,'********** Benchmarking *************'
    print >>logfile,''
    print >>logfile,'Total wall clock time was: %8.3f s.' % (endTime - startTime)
    print >>logfile,'Total CPU        time was: %8.3f s.' % (endProc - startProc)
    print >>logfile,'Wall processing  rate was: %8.3f MB/s.' % (17896.0 /
                                                                (endTime - startTime))
    
    ### Get last modification time of .ms.
    msfstat = os.stat(project+'.ms')
    print >>logfile,'* Breakdown:                           *'
    print >>logfile,'*  generating visibilities took %8.3fs,' % (msfstat[8] - startTime)
    print >>logfile,'*  deconvolution with %d iterations took %8.3fs.' % ( niter,
                                                                           endTime - msfstat[8])
    print >>logfile,'*************************************'
        
    logfile.close()
    
    print '-- Finished simdata test'
